<?php

namespace Myriad\EventDrivenService\Models;

use App\Interfaces\IUserFilterable;
use Carbon\Carbon;
use Gate;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Http\Resources\Json\JsonResource;

abstract class EventModel extends UUIDModel {
  use SoftDeletes;

  protected $commands = ['create', 'update', 'delete'];

  public final function getCommands() {
    return $this->commands;
  }

  public function format() {
    return new JsonResource($this);
  }

  public function getUpdated($lastUpdate, $perPage) {
    if (!isset($perPage)) {
      $perPage = $this->getPerPage();
    }

    $query = $this->select($this->getTable() . '.*')->withTrashed();
    if ($lastUpdate) {
      $lastUpdate = Carbon::createFromTimestamp($lastUpdate)->toDateTimeString();
      $query->where($this->getTable() . '.updated_at', '>', $lastUpdate);
    }

    if ($this instanceof IUserFilterable) {
      $query->filterForUser();
    }

    return $query->orderBy($this->getTable() . '.updated_at')->paginate($perPage);
  }

  public function createModel($event) {
    abort_unless(Gate::allows('create', [$this, $event['value']]), 403, 'Unauthorized to create model.');
    $this->fill($event['value'])->save();
  }

  public function updateModel($event) {
    abort_unless(Gate::allows('update', [$this, $event]), 403, 'Unauthorized to update model attribute.');
    $this->{$event['name']} = $event['value'];
    $this->{$event['name'] . '_version'} = $event['version_new'];
    $this->save();
  }

  public function deleteModel($event) {
    abort_unless(Gate::allows('delete', $this), 403, 'Unauthorized to delete model.');
    $this->delete();
  }

  public final function isCurrent($name, $version) {
    $currentVersion = $this[$name . '_version'];
    return $currentVersion == null || $currentVersion == $version;
  }
}